"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var mockery_1 = require("./mockery");
describe('Mockery', function () {
    var Foo = (function () {
        function Foo() {
            this.array = [];
            this.string = ':-)';
            this.booleanFunction = function () { return true; };
            this.functionWithParam = function (par) { return par; };
            this.objectFunction = function () { return ({
                anotherNestedObject: { function: function () { return true; } },
                string: 'hi',
                stringFunction: function () { return 'hi'; }
            }); };
            this.stringFunction = function (buzz) { return buzz.toUpperCase(); };
            this.voidFunction = function () { return undefined; };
        }
        Foo.static = function () {
            throw new Error();
        };
        return Foo;
    }());
    describe('of', function () {
        it('mocks the supplied method', function () {
            var mock = mockery_1.Mockery.of({ stringFunction: function () { return 'hi'; } });
            expect(mock.stringFunction('bye')).toEqual('hi');
        });
        it('mocks multiple methods', function () {
            var mock = mockery_1.Mockery.of({ array: [{ stringFunction: function () { return 'string'; } }],
                booleanFunction: function () { return false; },
                stringFunction: function () { return 'hi'; } });
            expect(mock.stringFunction('whatevs')).toBe('hi');
            expect(mock.booleanFunction()).toBe(false);
        });
        it('adds a spy to the supplied method on the mock object', function () {
            var mock = mockery_1.Mockery.of({ stringFunction: function () { return 'hi'; } });
            mock.stringFunction('bye');
            expect(mock.stringFunction).toHaveBeenCalledWith('bye');
        });
        it('works with no arguments', function () {
            expect(function () { return mockery_1.Mockery.of(); }).not.toThrow();
        });
        it('works with nested types', function () {
            var mock = mockery_1.Mockery.of({ nestedObject: { stringFunction: function () { return 'hi'; } } });
            expect(mock.nestedObject.stringFunction()).toEqual('hi');
        });
        it('mocks multiple level nested functions', function () {
            var mock = mockery_1.Mockery.of({ nestedObject: { anotherNestedObject: { function: function () { return false; } } } });
            mock.nestedObject.anotherNestedObject.function();
            expect(mock.nestedObject.anotherNestedObject.function).toHaveBeenCalled();
        });
        it('mocks partials function return types', function () {
            var mock = mockery_1.Mockery.of({ objectFunction: function () { return ({ string: 'bah' }); } });
            expect(mock.objectFunction().string).toBe('bah');
        });
    });
    describe('extend', function () {
        it('can extend an empty mock object', function () {
            var mock = mockery_1.Mockery.of();
            mockery_1.Mockery.extend(mock).with({ stringFunction: function () { return 'foooo'; } });
            expect(mock.stringFunction('bye')).toEqual('foooo');
        });
        it('extends the mock object with the supplied property', function () {
            var mock = mockery_1.Mockery.of();
            mockery_1.Mockery.extend(mock).with({ string: ':-)' });
            expect(mock.string).toEqual(':-)');
        });
        it('extends the mock object with spies for the supplied function', function () {
            var mock = mockery_1.Mockery.of();
            mockery_1.Mockery.extend(mock).with({ booleanFunction: function () { return false; } });
            mock.booleanFunction();
            expect(mock.booleanFunction).toHaveBeenCalled();
        });
        it('overrides preexisting mock', function () {
            var mock = mockery_1.Mockery.of({ stringFunction: function () { return 'hi'; } });
            mockery_1.Mockery.extend(mock).with({ stringFunction: function () { return 'foooo'; } });
            expect(mock.stringFunction('bye')).toEqual('foooo');
        });
        it('mocks an already mocked method multiple times', function () {
            var mock = mockery_1.Mockery.of({ booleanFunction: function () { return false; } });
            expect(mock.booleanFunction()).toBeFalsy();
            mockery_1.Mockery.extend(mock).with({ booleanFunction: function () { return true; } });
            expect(mock.booleanFunction()).toBeTruthy();
            mockery_1.Mockery.extend(mock).with({ booleanFunction: function () { return false; } });
            expect(mock.booleanFunction()).toBeFalsy();
            expect(mock.booleanFunction).toHaveBeenCalled();
        });
        it('mocks an already mocked property multiple times', function () {
            var mock = mockery_1.Mockery.of({ string: 'first' });
            expect(mock.string).toEqual('first');
            mockery_1.Mockery.extend(mock).with({ string: 'second' });
            expect(mock.string).toEqual('second');
            mockery_1.Mockery.extend(mock).with({ string: 'third' });
            expect(mock.string).toEqual('third');
        });
        it('mocks partials function return types', function () {
            var mock = mockery_1.Mockery.of({ objectFunction: function () { return ({ string: 'bah' }); } });
            expect(mock.objectFunction().string).toBe('bah');
        });
    });
    describe('staticMethod', function () {
        it('does not call the underlying implememtation', function () {
            mockery_1.Mockery.staticMethod(Foo, 'static', function () { return 'hi'; });
            expect(function () { Foo.static(); }).not.toThrow();
        });
        it('calls fake', function () {
            mockery_1.Mockery.staticMethod(Foo, 'static', function () { return 'hi'; });
            expect(Foo.static()).toEqual('hi');
        });
        it('is a spy', function () {
            mockery_1.Mockery.staticMethod(Foo, 'static', function () { return 'hi'; });
            Foo.static();
            expect(Foo.static).toHaveBeenCalled();
        });
        it('resets the call count', function () {
            mockery_1.Mockery.staticMethod(Foo, 'static', function () { return 'hi'; });
            Foo.static();
            mockery_1.Mockery.staticMethod(Foo, 'static', function () { return 'hi'; });
            expect(Foo.static).not.toHaveBeenCalled();
        });
        it('overwrites spy fake', function () {
            mockery_1.Mockery.staticMethod(Foo, 'static', function () { return 'hi'; });
            mockery_1.Mockery.staticMethod(Foo, 'static', function () { return 'hello'; });
            expect(Foo.static()).toEqual('hello');
        });
        it('calls the underlying implememtation', function () {
            expect(function () { Foo.static(); }).toThrow();
        });
    });
});
//# sourceMappingURL=mockery.spec.js.map